'''
Created on 31 Oct 2016

@author: jdrumgoole
'''

from audit import Audit
from feedback import Feedback
from pprint import pprint
from mugdata import MUGData
import itertools
from utils.query import Query

class Events(MUGData):
    '''
    classdocs
    '''
    
    def __init__(self, mdb, collection_name  ):
        '''
        Constructor
        '''
        super( Events, self ).__init__( mdb, collection_name ) 
    
    def get_all_events(self, query ={}):
        return self.find( query )
    def get_all_group_events(self, groups=[ "all" ] ):
        
        if "all" in groups:
            return self.find()
        else:
            return itertools.chain( *[ self.get_group_events( i )  for i in groups ] )
        
    def get_group_events(self, url_name ):
        return self.find( { "event.group.urlname" : url_name } )
    
#     def get_groups_events(self, groups=[] ):
#         # Groups should be an iterator
#         return itertools.chain( *[ self.get_group_events( i )  for i in groups ] )

    def summary( self, doc ):
        event  = doc[ "event"]
        group  = event[ "group" ]
        return u"name: {0}\ngroup: {1}\nrsvp:{2}\ntime: {3}\n".format(  event[ "name"], 
                                                                        group[ "urlname" ], 
                                                                        event[ "yes_rsvp_count"], 
                                                                        event[ "time" ] )
    
    def one_line(self, doc ):
        event  = doc[ "event"]
        group  = event[ "group" ]
        return u"name: {0}, date: {1}, group: {2}".format(  event[ "name"], 
                                                            event[ "time" ],
                                                            group[ "urlname" ],)

        
class PastEvents(Events):
    '''
    classdocs
    '''
    def __init__(self, mdb ):
        '''
        Constructor
        '''
        super( PastEvents, self ).__init__( mdb, "past_events")
        
    def get_all_group_events(self, groups=[ "all" ] ):
        
        if "all" in groups:
            return self.find({ "event.status" : "past"})
        else:
            return itertools.chain( *[ self.get_group_events( i )  for i in groups ] )
        
    def get_group_events(self, url_name ):
        return self.find( { "event.group.urlname" : url_name,
                            "event.status" : "past" } )
    
class UpcomingEvents(Events):
    '''
    classdocs
    '''


    def __init__(self, mdb ):
        '''
        Constructor
        '''
        super( UpcomingEvents, self ).__init__( mdb, "upcoming_events")
        
    def get_all_group_events(self, groups=[ "all" ] ):
        
        if "all" in groups:
            return self.find( { "event.status" : "ucpoming"})
        else:
            return itertools.chain( *[ self.get_group_events( i )  for i in groups ] )
        
    def get_group_events(self, url_name ):
        return self.find({ "event.group.urlname" : url_name,
                           "event.status"        : "upcoming" } )


'''
Created on 21 Mar 2017

@author: jdrumgoole
'''
import datetime

class Nested_Dict( dict ):
    '''
    classdocs
    '''    
    def __init__(self, *args, **kwargs):
        self.update(*args, **kwargs)

    def __getitem__(self, key):
        '''
        Accept a key of the form "a.b.c". We expect the object to have a set of nested dicts
        { a : ....
              b : { ....
                    c: ....
                }
            }
        } 
        
        If a,b and c exist as keys return the value of c.
        If any of the nested keys doesn't exist return value error.
               
        '''
        keys = key.split( ".", 1 )
        
        print( "getitem: %s" % key )
        if len( keys ) == 1 :
            print( "len(keys) : 1")
            print( "self: %s" % self )
            return dict.__getitem__( self, keys[ 0 ])
        else:
            #for k,v in dict.__iteritems__( self ):
            print( "len(keys) : > 1")
            nested = dict.__getitem__(  self, keys[ 0 ] )
            if isinstance( nested, dict ) :
                    return Nested_Dict.__getitem__( Nested_Dict( nested ), keys[ 1 ] )
                    

    def __setitem__(self, key, val):
        
        keys = key.split( ".", 1 )
        
        print( "setitem: %s" % key )
        if len( keys ) == 1 :
            print( "len(keys) : 1")
            print( "self: %s" % self )
            return dict.__setitem__( self, keys[ 0 ], val )
        else:
            #for k,v in dict.__iteritems__( self ):
            print( "len(keys) : > 1")
            if isinstance( dict.__getitem__(  self, keys[ 0 ] ), dict ) :
                    return Nested_Dict.__setitem__( Nested_Dict.__getitem__( self, keys[ 0 ] ), keys[ 1 ], val )

    def __repr__(self):
        dictrepr = dict.__repr__(self)
        return '%s(%s)' % (type(self).__name__, dictrepr)
    
    def update(self, *args, **kwargs ):
        print 'update', args, kwargs
        for k, v in dict(*args, **kwargs).iteritems():
            self[k] = v
        
def get_value( d, k ):
    keys = k.split( ".", 1 )
    
    print( "get_value %s" % k  )
    if len( keys ) == 1 :
        print( "len(keys) : 1")
        return d[ keys[ 0 ]]
    else:
        #for k,v in dict.__iteritems__( self ):
        print( "len(keys) : > 1")
        nested = d[ keys[ 0 ]]
        if isinstance( nested, dict ) :
                return get_value( nested, keys[ 1 ] )
                

def set_value( d, k, v ):
    
    keys = k.split( ".", 1 )
    
    if len( keys ) == 1 :
        d[ keys[ 0 ]] = v
    else:
        set_value( d[ keys[0]], keys[ 1 ], v )
        
if __name__ == "__main__" :
    print( "hello")
    
    nested = { "a" : 
              { "b" : 
               { "c" : datetime.datetime.now() }}}

    d = Nested_Dict(nested )
    
    print( get_value( d,  'a.b.c'))
    print( get_value( d,  'a.b'))
    print( get_value( d,  'a' ))
    
    nested = { "x" : 10,
               "a" : 
              { "b" : 
               { "c" : datetime.datetime.now() }}}
    
    d = Nested_Dict(nested )
    
    print( get_value( d, 'a.b.c'))
    print( get_value( d, 'a.b' ))
    print( get_value( d, 'a' ))
    print( get_value( d, 'x' ))
    
    
    print( set_value( d, 'x', 12 ))
    
    set_value( d,  'a.b.c', "hello" )
    print( d )


'''
Created on 1 Nov 2016

@author: jdrumgoole
'''

from .mugdata import MUGData
        
class Attendees( MUGData ):
    
    def __init__(self, mdb ):
        '''
        Constructor
        '''
        super( Attendees, self ).__init__( mdb, "attendees")   
    
    def get_attendees(self, event_url):
        
        return self.find( { "info.event.event_url" : event_url })

        
    def get_all_attendees(self , q=None):
        
        return self.find( q )
    
    def summary( self, doc ):

        event = doc[ "info"][ "event" ]
        address = ""
        city = ""
        country = ""
        
        headline = self.oneline( doc )
        if "venue" in event:
            address = event[ "venue" ][ "address_1" ]
            city = event[ "venue"][ "city"]
            country = event[ "venue" ][ "country"]
            
        return "%s\n\taddress:%s\n\tcity:%s\n\tcountry: %s" % ( headline,
                                                                address,
                                                                city,
                                                                country )
        
    def oneline(self, doc ):
        member = doc[ "info" ][ "attendee"][ "member" ]
        event = doc[ "info"][ "event" ]
        
        return "name: %s, event:%s, date: %s, rsvp: %i" % ( member[ "name" ], 
                                                            event[ "name"], 
                                                            event[ "time" ],
                                                            event[ "yes_rsvp_count"])



'''
Created on 21 Nov 2016

@author: jdrumgoole
'''

from mugalyser.batchwriter import BatchWriter
from requests import HTTPError
import logging
from mugalyser.apikey import get_meetup_key
from mugalyser.meetup_api import MeetupAPI


def mergeEvents( writer ):
    for attendee, event in writer:
        doc = { u"attendee" : attendee,
                u"event" : event }
        yield doc 
        
class MeetupWriter(object):
    '''
    A class that reads data about MUGS from the Meetup API using the MeetupAPI class and writes that
    data to a MongoDB collection. Supports pro and no pro APIs
    '''
    def __init__(self, audit, mdb, urls, apikey= get_meetup_key(), unordered=True ):
        '''
        Write contents of meetup API to MongoDB
        '''

        self._mdb = mdb
        self._meetup_api = MeetupAPI( apikey )
        self._audit = audit
        self._groups = self._mdb.groupsCollection()
        self._members = self._mdb.membersCollection()
        self._attendees = self._mdb.attendeesCollection()
        self._pastEvents = self._mdb.pastEventsCollection()
        self._upcomingEvents = self._mdb.upcomingEventsCollection()
        self._mugs = []
        self._unordered = unordered
        self._urls = urls
        
        
    def process(self, collection, retrievalGenerator, processFunc, newFieldName ):
        '''
        Call batchWriter with a collection. Use retrievalGenerator to get a single
        document (this should be a generator function). Use processFunc to tranform the 
        document into a new doc (it should take a doc and return a doc).
        Write the new doc using the newFieldName.
        
        Write is done using a generator as well. The write receiver accumulates writes until a threshold
        is reached and then writes them as a batch using BatchWriter.
        
        '''
        bw = BatchWriter( collection, processFunc, newFieldName, orderedWrites=self._unordered )
        writer = bw.bulkWrite( writeLimit=1)
        
        for i in retrievalGenerator :
            writer.send( i )

    
    def processAttendees( self, group ):
        
        writer = self._meetup_api.get_attendees( group, items=100)
        
        newWriter = mergeEvents( writer )
        self.process( self._attendees, newWriter, self._audit.addTimestamp, "info"  )
        
    def processGroup(self, url_name, groupName="group"):
        group = self._meetup_api.get_group( url_name )
        newDoc = self._audit.addTimestamp( groupName, group )
        self._groups.insert_one( newDoc )
        return newDoc

    def updateGroup(self, groupName, doc ):
        self._mugs.append( doc[ "urlname" ])
        return self._audit.addTimestamp( groupName, doc )
        
    def processGroups(self, nopro ):
        if nopro:
            groups = self.get_groups()
        else:
            groups = self._meetup_api.get_pro_groups()
            
        self.process( self._groups,  groups, self.updateGroup, "group" )
        
    def get_groups(self ):
        for i in self._urls:
            yield self._meetup_api.get_group( i )
        
    def processPastEvents(self, url_name ):
        pastEvents = self._meetup_api.get_past_events( url_name )
        self.process( self._pastEvents, pastEvents, self._audit.addTimestamp, "event" )
   
    def processUpcomingEvents(self, url_name ):
        upcomingEvents = self._meetup_api.get_upcoming_events( url_name )
        self.process( self._upcomingEvents, upcomingEvents, self._audit.addTimestamp, "event" )
        
    def processMembers( self, nopro=True ):
        
        if nopro:
            members = self.get_members()
        else:
            members = self._meetup_api.get_pro_members()
            
        self.process( self._members, members, self._audit.addTimestamp, "member" )
        
    def get_members(self ):
        for i in self._urls:
            return self._meetup_api.get_members( i )
        
        return []
            
    def mug_list(self):
        return self._mugs
    
    
    def capture_snapshot(self, url_name, nopro, admin_arg, phases ):

        try :
        
            for i in phases:
                if i == "pastevents" :
                    logging.info( "process past events for      : '%s'", url_name )
                    self.processPastEvents( url_name )
                elif i == "upcomingevents" :
                    logging.info( "process upcoming events for  : '%s'", url_name )
                    self.processUpcomingEvents( url_name )
                elif i == "attendees" :
                    if admin_arg:
                        logging.info( "process attendees            : '%s'", url_name )
                        self.processAttendees( url_name )
                    else:
                        logging.warn( "You have not specified the admin arg")
                        logging.warn( "You must be a meetup admin user to request attendees")
                        logging.warn( "Ignoring phase 'attendees")
            
                else:
                    logging.warn( "ignoring phase '%s': not a valid execution phase", i )
    
        except HTTPError, e :
            logging.fatal( "Stopped processing: %s", e )
            raise








'''
Created on 4 Apr 2017

@author: jdrumgoole
'''
    
class Nested_Dict( object ):
    '''
    Allow dotted access of nested dicts.
    so :
    a[ "x.y.z" ] = 1 is equivalent to a[ x[ y[ z ]]] = 1
    
    a = { x : { y : z }}}
    '''
    
    def __init__(self, d={}):
        
        if isinstance( d, dict ):
            self._dict = d
        else:
            raise ValueError( "d is not a dict type")

    def dict_value(self ):
        return self._dict
    
    def get_value( self, k ):
        keys = k.split( ".", 1 )
        
        #print( "get_value %s" % k  )
        if len( keys ) == 1 :
            #print( "len(keys) : 1")
            return self._dict[ keys[ 0 ]]
        elif self._dict.has_key( keys[ 0 ]) :
                nested = self._dict[ keys[ 0 ]]
        else:
            raise ValueError( "nested key :'%s' does not exist in keys %s of %s" % ( keys[ 0 ], keys, self._dict ))
        
        if isinstance( nested, dict ) :
            nested = Nested_Dict( nested )
            return nested.get_value( keys[ 1 ] )
        else:
            return nested
    
    def has_key( self, k ):
        
        keys = k.split( ".", 1 )
        nested = None
        if len( keys ) == 1 :
            return self._dict.has_key( keys[ 0 ] )
        elif self._dict.has_key( keys[ 0 ]) :
                nested = self._dict[ keys[ 0 ]]
        else:
            raise ValueError( "nested key :'%s' does not exist" % keys[ 0 ])
        
        if isinstance( nested, dict ) :
            nested = Nested_Dict( nested )
            return nested.has_key( keys[ 1 ] )
        else:
            return True
                      
        
    def set_value( self, k, v ):
        
        keys = k.split( ".", 1 )
        nested = None
        if len( keys ) == 1 :
            self._dict[ keys[ 0 ]] = v
            return self
        elif self._dict.has_key( keys[ 0 ]) :
            nested = self._dict[ keys[ 0 ]]
        else:
            self._dict[ keys[ 0 ]] = {}
            nested = self._dict[ keys[ 0 ]]
        
        if isinstance( nested, dict ) :
            nested = Nested_Dict( nested )
            nested.set_value( keys[ 1 ], v )
            return self



class dotted_dict( dict ):
    '''
    Allow dotted access of nested dicts.
    so :
    a[ "x.y.z" ] = 1 is equivalent to a[ x[ y[ z ]]] = 1
    
    a = { x : { y : z }}}
    '''
    def __init__( self, *args, **kwargs ):
        super( dotted_dict, self).__init__( *args, **kwargs )


    def dict_value(self ):
        return self._dict
    
    def get_value( self, k ):
        keys = k.split( ".", 1 )
        
        #print( "get_value %s" % k  )
        if len( keys ) == 1 :
            #print( "len(keys) : 1")
            return self._dict[ keys[ 0 ]]
        elif self._dict.has_key( keys[ 0 ]) :
                nested = self._dict[ keys[ 0 ]]
        else:
            raise ValueError( "nested key :'%s' does not exist in keys %s of %s" % ( keys[ 0 ], keys, self._dict ))
        
        if isinstance( nested, dict ) :
            nested = NestedDict( nested )
            return nested.get_value( keys[ 1 ] )
        else:
            return nested
    
    def has_key( self, k ):
        
        keys = k.split( ".", 1 )
        nested = None
        if len( keys ) == 1 :
            return self._dict.has_key( keys[ 0 ] )
        elif self._dict.has_key( keys[ 0 ]) :
                nested = self._dict[ keys[ 0 ]]
        else:
            raise ValueError( "nested key :'%s' does not exist" % keys[ 0 ])
        
        if isinstance( nested, dict ) :
            nested = NestedDict( nested )
            return nested.has_key( keys[ 1 ] )
        else:
            return True
                      
        
    def set_value( self, k, v ):
        
        keys = k.split( ".", 1 )
        nested = None
        if len( keys ) == 1 :
            self._dict[ keys[ 0 ]] = v
            return self
        elif self._dict.has_key( keys[ 0 ]) :
            nested = self._dict[ keys[ 0 ]]
        else:
            self._dict[ keys[ 0 ]] = {}
            nested = self._dict[ keys[ 0 ]]
        
        if isinstance( nested, dict ) :
            nested = NestedDict( nested )
            nested.set_value( keys[ 1 ], v )
            return self


'''
Created on 14 Oct 2016

@author: jdrumgoole
'''

import pymongo
import csv
import sys
from argparse import ArgumentParser
from mongodb import MUGAlyserMongoDB
from members import Members
from audit import Audit
import pprint
try:
    from apikey import get_meetup_key
except ImportError,e :
    print( "Failed to import apikey: have you run makeapikeyfile_main.py <APIKEY> : %s" % e )
    sys.exit( 2 )

if __name__ == '__main__': 
        
    parser = ArgumentParser()
        
    parser.add_argument( "--file", default="mongodb_members_16_nov.csv", help="Input file" )

    args = parser.parse_args()
    
    mdb = MUGAlyserMongoDB( "mongodb://localhost" )
    
    members = Members( mdb, get_meetup_key()).collection()
    audit = Audit( mdb ).collection() 
    
    count = 0
    notFoundCount = 0
    foundCount = 0
    csvmembers={}
    lastBatchID = Audit( mdb ).getLastBatchID()
    
    print( "Processing batch ID: %i" % lastBatchID )
    with open( args.file, "rb") as csvdata :
        with open( "csvfile.out", "w" ) as csvout:
            reader = csv.DictReader( csvdata, delimiter="," )
            for r in reader:
                #print( r )
                count = count + 1
                csvmembers[ r[" member_id" ]] = r 
                q = { "member.id" :  int( r[ " member_id" ] ), "batchID" : lastBatchID }
                #pprint.pprint( "query: %s" % q)
                if members.find_one( { "member.id" :  int( r[ " member_id" ] ), "batchID" : lastBatchID } ) :
                    #print( "%i. %s found" % ( count, r["member_name"] ))
                    foundCount = foundCount + 1
                    csvout.write( "found in db: '%s'\n" % r[" member_id"])
                else:
                    print( "%i. %s %s not found ********************" % (count, r[ " member_id" ], r[ "member_name"] ))
                    csvout.write( "not found in db: '%s %s'\n" % ( r[ " member_id"], r[ "member_name"]))
                    notFoundCount = notFoundCount + 1
                    
        
    countCSV = 0
    missing = []
    with open( "dbmembers", "w" ) as dbout:
        for i in members.find( {"batchID" : lastBatchID } ):
    
            countCSV = countCSV + 1
            if "member" in i:
                if i[ "member" ][ "id" ] in csvmembers :
                    dbout.write( (u"in csv : %s\n" % i["member"]["id"]).encode('utf-8'))
                else:
                    dbout.write( ( u"not in csv : %s\n" % i["member"]["id"]).encode('utf-8'))
                    missing.append( i["member"]["id"] )   
            else:
                print( "no member field" )
                    
            #print( "processed: %i" % countCSV )
         
    print( "Not found in CSV file: %i" % notFoundCount )   
    print( "found in CSV file: %i" % foundCount )
    print( "Total CSV docs: %i" % countCSV )
    print( "Total db docs : %i" % count )
    print( "Total Db missing : %i" % len( missing )) 
    
        
        


'''
Created on 22 Jun 2016

@author: jdrumgoole
'''

import pymongo

class MUGAlyserMongoDB( object ):
    
    def __init__(self, uri="mongodb://localhost:27017/", database="MUGS",  setup=True):

    #def __init__(self, host="localhost", port=27017, databaseName="MUGS", replset="",
    #            username=None, password=None, ssl=False, admin="admin", connect=True):
        
        '''
        Example URL 
        
        mongodb://<username>:<password>@<host list>/database?<args>
        
        "mongodb://jdrumgoole:PASSWORD@mugalyser-shard-00-00-ffp4c.mongodb.net:27017,
        mugalyser-shard-00-01-ffp4c.mongodb.net:27017,
        mugalyser-shard-00-02-ffp4c.mongodb.net:27017/admin?ssl=true&replicaSet=MUGAlyser-shard-0&authSource=admin"
        
        '''
        
        self._uri             = uri 
        self._client          = None
        self._members         = None
        self._groups          = None
        self._pastEvents      = None
        self._upcomingEvents  = None
        self._attendees       = None
        
        
        if setup:
            self.setup()

        
    def setup(self ):

        if self._uri.startswith( "mongodb://" ) :
            self._client = pymongo.MongoClient( host=self._uri, tz_aware=True )
        else:
            raise ValueError( "Invalid URL: %s" % self._uri )
        
        self._database = self._client.get_default_database()
        
#         if self._username :
#             #self._admindb = self._client[ self._admin ]
#             if self._database.authenticate( name=self._username, password=self._password, source=self._admin ):
# #            if self._database.authenticate( self._username, self._password, mechanism='MONGODB-CR'):
#                 logging.debug( "successful login by %s (method SCRAM-SHA-1)", self._username )
#             else:
#                 logging.error( "login failed for %s (method SCRAM-SHA-1)", self._username )
                
        self._members         = self._database[ "members" ]
        self._groups          = self._database[ "groups" ]
        self._pastEvents      = self._database[ "past_events" ]
        self._upcomingEvents  = self._database[ "upcoming_events" ]
        self._audit           = self._database[ "audit" ]
        self._attendees       = self._database[ "attendees"]
        
        self._audit.create_index( [("name", pymongo.ASCENDING )] )
        
        self._members.create_index([("member.location", pymongo.GEOSPHERE)])
        self._members.create_index([("member.name", pymongo.ASCENDING )])
        self._members.create_index([("member.id", pymongo.ASCENDING )])
        self._members.create_index([( "batchID", pymongo.ASCENDING )])
        self._members.create_index([( "member.join_time", pymongo.ASCENDING )])
        self._members.create_index([( "member.last_access_time", pymongo.ASCENDING )])
                
        self._groups.create_index([( "batchID", pymongo.ASCENDING )])
        self._pastEvents.create_index([( "batchID", pymongo.ASCENDING )])
        self._upcomingEvents.create_index([( "batchID", pymongo.ASCENDING )])
        self._attendees.create_index([( "batchID", pymongo.ASCENDING )])
        
    def client(self):
        return self._client
    
    def database(self) :
        return self._database
    
    def make_collection(self, collection_name ):
        return self._database[ collection_name ]
    
    def collection_stats(self, collection_name ):
        return self._database.command( "collstats", collection_name )
    
    def pastEventsCollection(self):
        return self._pastEvents
    
    def upcomingEventsCollection(self):
        return self._upcomingEvents
    
    def groupsCollection(self):
        return self._groups

    def auditCollection(self):
        return self._audit
    
    def membersCollection(self):
        return self._members
    
    def attendeesCollection(self):
        return self._attendees
    
    def collection_names(self ):
        return [ "groups" , "members", "attendees", "past_events" , "upcoming_events" ]


'''
Created on 29 Mar 2017

@author: jdrumgoole
'''
from argparse import ArgumentParser


def make_mongodb_parser( args, parser ):
    
    parser = ArgumentParser( args, parents=[ parser ] )
        
    parser.add_argument( "--host", default="mongodb://localhost:27017/MUGS", 
                         help="URI for connecting to MongoDB [default: %(default)s]" )
    
    return parser


'''
Created on 30 Sep 2016

This is an attempt at a singleton. 

The Audit collection keeps track of all the runs of the MUGALyser tool.
There is a master document which tracks the current batch. Here is an
example.

{  "name"          : "Current Batch"
   "currentID"     : 13
   "batchID"       : 0
   "schemaVersion" : 0.8
   "timestamp"     : October 10, 2016 9:16 PM }
   
Name is used to identify the document.
currentID is the current batch ID. i.e. the last batch to be processed.
Timestamp tells us the date and time on which that batch started being 
processed.

batchID allows us to identify the CurrentBatch record and as batchID is
indexed this is a fast way to find the CurrentBatch record.

If we now look at the corresponding batch document.

{ "batchID" :  13
  "start"    : October 10, 2016 9:16 PM
  "info"     : { "args"  : { ... }
                 "MUGS" : { ... }
                }
   "version" : "mugalyser_main.py 0.7 beta"
   "trial"   : false
   "end"     : October 10, 2016 9:42 PM
}

This tells us that this is the data for batchID 13.

The run started at "start" time. Which corresponds to the timestamp
in "CurrentBatch" above.

The info field contains the command line arguments and the list of MUGS
that were processed. The information is elided here for brevity (do take a look
at the schema after a run).

"version" is the version of the MUGAlyser program that captured this data.

"trial" : Indicates this was a trial run so no data was captured (used
for testing).

"end" : Indicates when the run completed. An incomplete run will not 
have an end date field.

@author: jdrumgoole
'''

from datetime import datetime
from version import __version__, __schemaVersion__

from apikey import get_meetup_key

class AuditSingleton( object ):
    
    class __Audit_Impl( object ):
        name="audit"
        
        def __init__(self, mdb ):
            
            self._mdb = mdb
            self._auditCollection = mdb.auditCollection()
            self._currentBatch = self._auditCollection.find_one( { "name" : "Current Batch"})
            
            if self._currentBatch is None:
                self._currentBatch = {}
                self._currentBatch[ "name"] = "Current Batch"
                self._currentBatch[ "currentID" ] = 0
                self._currentBatch[ "batchID" ] = 0
                self._currentBatch[ 'valid'] = False
                self._currentBatch[ "schemaVersion" ] = __version__
                self._auditCollection.insert_one( self._currentBatch )
            else:
                # Migrate schema from version 0.7 to 0.8
                if self._currentBatch.has_key( "ID" ):
                    self._auditCollection.update( { "_id" : self._currentBatch[ "_id"]},
                                                  { "$rename" : { "ID" : "currentID" }})
                    
                    curid = self._currentBatch[ "ID"]
                    del self._currentBatch[ "ID"]
                    self._currentBatch[ "currentID" ]= curid
                    
                if not "batchID" in self._currentBatch :
                    self._auditCollection.update( { "_id" : self._currentBatch[ "_id"]},
                                                  { "$set" : { "batchID" : 0 }})
                
                    self._currentBatch[ "batchID" ] = 0
                
                self._auditCollection.update( { "_id" : self._currentBatch[ "_id"]},
                                              { "$set" : { "schemaVersion" : __schemaVersion__  }})
    
            self._currentBatchID = None
            
        def collection(self):
            return self._auditCollection
            
        def mdb( self ):
            return self._mdb
        
        def inBatch(self):
            return self._currentBatchID
        
        def insertTimestamp( self, doc, ts=None ):
            if ts :
                doc[ "timestamp" ] = ts
            else:
                doc[ "timestamp" ] = datetime.utcnow()
                
            return doc
        
        def addTimestamp( self, name, doc, ts=None ):
        
            tsDoc = { name : doc, "timestamp" : None, "batchID": self.getCurrentBatchID()}
            return self.insertTimestamp( tsDoc, ts )
        
        def addInfoTimestamp(self, doc, ts=None ):
            return self.addTimestamp( "info", doc, ts )
        
        def addMemberTimestamp(self, doc, ts=None ):
            return self.addTimestamp( "member", doc, ts )
        
        def addEventTimestamp(self, doc, ts=None ):
            return self.addTimestamp( "event", doc, ts )
        
        def addGroupTimestamp(self, doc, ts=None ):
            return self.addTimestamp( "group", doc, ts )
        
        def addAttendeeTimestamp(self, doc, ts=None ):
            return self.addTimestamp( "attendee", doc, ts )
        
        def getBatchIDs(self):
            cursor = self._auditCollection.find( { "batchID" : { "$exists" : 1 }}, { "_id" : 0, "batchID" : 1})
            for i in cursor:
                if i[ "batchID"] == 0 :
                    continue
                yield i[ 'batchID' ]
                
        def incrementBatchID(self):
            #
            # We can have multiple batches running in parallel as long as each has a unique
            # batch ID. Find And Modify ensures that each batch ID is unique.
            curBatch = self._auditCollection.find_and_modify( { "name" : "Current Batch" },
                                                              update= { "$inc" : { "currentID" : 1 },
                                                                        "$set" : { "timestamp" : datetime.now() }},
                                                             new = True )
            
            return curBatch[ "currentID" ]
    #         self._currentBatch[ "currentID" ] = self.getCurrentBatchID()  + 1
    #         self._currentBatch[ "timestamp" ] = datetime.now()
    #         self._auditCollection.update( { "name" : "Current Batch" }, 
    #                                       { "$set" : { "currentID" : self._currentBatch[ "currentID"],
    #                                                    "timestamp" : self._currentBatch[ "timestamp" ] }} )
            
        def startBatch(self, doc, name=None, trial=False, apikey = get_meetup_key()):
            thisBatchID = self.incrementBatchID()
            
            if name is None :
                name = "Standard Batch: %i" % thisBatchID
                
            self._auditCollection.insert_one( { "batchID" : thisBatchID,
                                                "start"   : datetime.now(),
                                                "trial"   : trial,
                                                "end"     : None,
                                                "name"    : name,
                                                "apikey"  : apikey,
                                                "info"    : doc })
            
            self._currentBatchID = thisBatchID
            
            return thisBatchID
            
        def getBatch(self, batchID ):
            return self._auditCollection.find_one( { "batchID" : batchID })
        
        def incomplete(self, batchID ):
            return self.getBatch( batchID )[ "end" ] == None
            
        def endBatch(self, batchID ):
            self._auditCollection.update( { "batchID" : batchID },
                                          { "$set" : { "end"  : datetime.now(), 
                                                      "valid" : True }})
            
            self._currentBatchID = None
            
        def auditCollection(self):
            return self._auditCollection
        
        def getLastBatchID(self):
            curBatch = self._auditCollection.find_one( { "name" : 'Current Batch'} )
            if curBatch[ "currentID" ] < 2 :
                raise ValueError( "No valid last batch")
            else:
                return curBatch[ "currentID"] - 1
        
        def getCurrentValidBatchID( self ):
            raise ValueError( "not implemented")
        
        def getCurrentBatchID(self ):
            if self._currentBatchID :
                return self._currentBatchID
            else:
                curBatch = self._auditCollection.find_one( { "name" : 'Current Batch'} )
                
                if curBatch[ "currentID" ] == 0 :
                    raise ValueError( "No batches in database" )
                else:
                    return curBatch[ "currentID" ]
        
    __instance = None

    def __init__(self, mdb ):
        """ Create singleton instance """
        # Check whether we already have an instance
        if Audit.__instance is None:
            # Create and remember instance
            Audit.__instance = Audit.__Audit_Impl( mdb )

        # Store instance reference as the only member in the handle
        self.__dict__['_Singleton__instance'] = Audit.__instance

    def __getattr__(self, attr):
        """ Delegate access to implementation """
        return getattr(self.__instance, attr)

    def __setattr__(self, attr, value):
        """ Delegate access to implementation """
        return setattr(self.__instance, attr, value)


'''
Created on 30 Sep 2016

The Audit collection keeps track of all the runs of the MUGALyser tool.
There is a master document which tracks the current batch. Here is an
example.

{  "name"          : "Current Batch"
   "currentID"     : 13
   "batchID"       : 0
   "schemaVersion" : 0.8
   "timestamp"     : October 10, 2016 9:16 PM }
   
Name is used to identify the document.
currentID is the current batch ID. i.e. the last batch to be processed.
Timestamp tells us the date and time on which that batch started being 
processed.

batchID allows us to identify the CurrentBatch record and as batchID is
indexed this is a fast way to find the CurrentBatch record.

If we now look at the corresponding batch document.

{ "batchID" :  13
  "start"    : October 10, 2016 9:16 PM
  "info"     : { "args"  : { ... }
                 "MUGS" : { ... }
                }
   "version" : "mugalyser_main.py 0.7 beta"
   "trial"   : false
   "end"     : October 10, 2016 9:42 PM
}

This tells us that this is the data for batchID 13.

The run started at "start" time. Which corresponds to the timestamp
in "CurrentBatch" above.

The info field contains the command line arguments and the list of MUGS
that were processed. The information is elided here for brevity (do take a look
at the schema after a run).

"version" is the version of the MUGAlyser program that captured this data.

"trial" : Indicates this was a trial run so no data was captured (used
for testing).

"end" : Indicates when the run completed. An incomplete run will not 
have an end date field.

@author: jdrumgoole
'''

import pymongo
from datetime import datetime
from mugalyser.version import __programName__, __version__, __schemaVersion__
from mugalyser.agg import Agg, Sorter
from mugalyser.apikey import get_meetup_key

class Audit( object ):
    
    name="audit"
    
    def __init__(self, mdb ):
        
        self._mdb = mdb
        self._auditCollection = mdb.auditCollection()
        self._currentBatch = self._auditCollection.find_one( { "name" : "Current Batch"})
        
        if self._currentBatch is None:
            self._currentBatch = {}
            self._currentBatch[ "name"] = "Current Batch"
            self._currentBatch[ "currentID" ] = 0
            self._currentBatch[ "batchID" ] = 0
            self._currentBatch[ 'valid'] = False
            self._currentBatch[ "schemaVersion" ] = __version__
            self._auditCollection.insert_one( self._currentBatch )
        else:
            # Migrate schema from version 0.7 to 0.8
            if self._currentBatch.has_key( "ID" ):
                self._auditCollection.update( { "_id" : self._currentBatch[ "_id"]},
                                              { "$rename" : { "ID" : "currentID" }})
                
                curid = self._currentBatch[ "ID"]
                del self._currentBatch[ "ID"]
                self._currentBatch[ "currentID" ]= curid
                
            if not "batchID" in self._currentBatch :
                self._auditCollection.update( { "_id" : self._currentBatch[ "_id"]},
                                              { "$set" : { "batchID" : 0 }})
            
                self._currentBatch[ "batchID" ] = 0
            
            self._auditCollection.update( { "_id" : self._currentBatch[ "_id"]},
                                          { "$set" : { "schemaVersion" : __schemaVersion__  }})

        self._currentBatchID = None
        
    def collection(self):
        return self._auditCollection
        
    def mdb( self ):
        return self._mdb
    
    def inBatch(self):
        return self._currentBatchID
    
    def isProBatch(self, ID ):
        doc = self.getBatch( ID )

        return ( doc.has_key( "info") and 
                 doc[ "info"].has_key( "pro_account" ) and 
                 ( doc[ "info" ][ "pro_account"] == True ))
              
    def insertTimestamp( self, doc, ts=None ):
        if ts :
            doc[ "timestamp" ] = ts
        else:
            doc[ "timestamp" ] = datetime.utcnow()
            
        return doc
    
    def addTimestamp( self, name, doc, ts=None ):
    
        tsDoc = { name : doc, "timestamp" : None, "batchID": self.getCurrentBatchID()}
        return self.insertTimestamp( tsDoc, ts )
    
    def addInfoTimestamp(self, doc, ts=None ):
        return self.addTimestamp( "info", doc, ts )
    
    def addMemberTimestamp(self, doc, ts=None ):
        return self.addTimestamp( "member", doc, ts )
    
    def addEventTimestamp(self, doc, ts=None ):
        return self.addTimestamp( "event", doc, ts )
    
    def addGroupTimestamp(self, doc, ts=None ):
        return self.addTimestamp( "group", doc, ts )
    
    def addAttendeeTimestamp(self, doc, ts=None ):
        return self.addTimestamp( "attendee", doc, ts )
    
    def getBatchIDs(self):
        cursor = self._auditCollection.find( { "batchID" : { "$exists" : 1 }}, { "_id" : 0, "batchID" : 1})
        for i in cursor:
            if i[ "batchID"] == 0 :
                continue
            yield i[ 'batchID' ]
            
    def incrementBatchID(self):
        #
        # We can have multiple batches running in parallel as long as each has a unique
        # batch ID. Find And Modify ensures that each batch ID is unique.
        curBatch = self._auditCollection.find_and_modify( { "name" : "Current Batch" },
                                                          update= { "$inc" : { "currentID" : 1 },
                                                                    "$set" : { "timestamp" : datetime.now() }},
                                                         new = True )
        
        return curBatch[ "currentID" ]
#         self._currentBatch[ "currentID" ] = self.getCurrentBatchID()  + 1
#         self._currentBatch[ "timestamp" ] = datetime.now()
#         self._auditCollection.update( { "name" : "Current Batch" }, 
#                                       { "$set" : { "currentID" : self._currentBatch[ "currentID"],
#                                                    "timestamp" : self._currentBatch[ "timestamp" ] }} )
        
    def startBatch(self, doc, name=None, trial=False, apikey = get_meetup_key()):
        thisBatchID = self.incrementBatchID()
        
        if name is None :
            name = "Standard Batch: %i" % thisBatchID
            
        self._auditCollection.insert_one( { "batchID" : thisBatchID,
                                            "start"   : datetime.now(),
                                            "trial"   : trial,
                                            "end"     : None,
                                            "name"    : name,
                                            "apikey"  : apikey,
                                            "info"    : doc })
        
        self._currentBatchID = thisBatchID
        
        return thisBatchID
        
    def getBatch(self, batchID ):
        return self._auditCollection.find_one( { "batchID" : batchID })
    
    def incomplete(self, batchID ):
        return self.getBatch( batchID )[ "end" ] == None
        
    def endBatch(self, batchID ):
        self._auditCollection.update( { "batchID" : batchID },
                                      { "$set" : { "end"  : datetime.now(), 
                                                  "valid" : True }})
        
        self._currentBatchID = None
        
    def auditCollection(self):
        return self._auditCollection
    
    def getLastBatchID(self):
        curBatch = self._auditCollection.find_one( { "name" : 'Current Batch'} )
        if curBatch[ "currentID" ] < 2 :
            raise ValueError( "No valid last batch")
        else:
            return curBatch[ "currentID"] - 1
    
    def getCurrentValidBatchID( self ):
        curBatch = self._auditCollection.find( { "apikey" : get_meetup_key(),
                                                 "end"    : { "$type" : "date" }, #17 BSON type for timestamp
                                                 "trial"  : False } ).sort( "batchID", pymongo.DESCENDING ).limit( 1 )
        
        if curBatch is None :
            raise ValueError( "No current valid batch ID" )
        else:
            try :
                x= curBatch.next()[ "batchID"]
                return x
            except StopIteration :
                raise ValueError( "Have you set a valid API key? (APIKEY='%s')" % get_meetup_key())
            
    def getCurrentValidBatches( self, start=None, end=None ):
        
        agg = Agg( self._auditCollection )
        agg.addMatch({ "apikey" : get_meetup_key(),
                       "end"    : { "$ne" : None },
                       "trial"  : False }  )
        
        agg.addRangeMatch( "start", start, end)
        agg.addSort( Sorter( batchID = pymongo.DESCENDING ))
        
#         cursor = self._auditCollection.find( { "apikey" : get_meetup_key(),
#                                                "end"    : { "$ne" : None },
#                                                "trial"  : False } ).sort( "batchID", pymongo.DESCENDING )
                                         
        for i in agg.aggregate() :
            yield i
            
    def getCurrentValidBatchIDs( self ):
        for i in self.getCurrentValidBatches():
            yield i[ "batchID" ]
            
    def getCurrentBatch( self ) :
        return self._auditCollection.find_one( { "name" : 'Current Batch'} )
    
    def getCurrentBatchID(self ):
        if self._currentBatchID :
            return self._currentBatchID
        else:
            curBatch = self._auditCollection.find_one( { "name" : 'Current Batch'} )
            
            if curBatch[ "currentID" ] == 0 :
                raise ValueError( "No batches in database" )
            else:
                return curBatch[ "currentID" ]
    


'''
Created on 1 Nov 2016

@author: jdrumgoole
'''
import logging
from version import __programName__
            
class Feedback(object):
    '''
    classdocs
    '''

    def __init__(self ):
        '''
        Constructor
        '''
        self._report = self.report()
        self._report.next()
        
    def report(self):
        while True:
            line = (yield)
            if line is None:
                break
            else:
                logging.info( line )
                #print( line )
                
    def output(self, line ):

        logging.info( line )
        
    



'''
Created on 6 Sep 2016

@author: jdrumgoole
'''
import meetup.api
import pymongo
import pprint
from pip._vendor.html5lib.trie import Trie
if __name__ == "__main__" :
    
    
    meetup_groups= { "London-MongoDB-User-Group"  : None, 
                     "DublinMUG"                  : None, 
                     "Paris-MongoDB-User-Group"   : None }
    
    meetup_API_key = "4a1e51774664e3412136d6457522120"
    
    client = meetup.api.Client( meetup_API_key )
    
    mclient = pymongo.MongoClient()
    db = mclient.MUGAudit
    mugs = db.MuGs
    groups = db.GroupState
    
    mongo_groups = client.GetGroups({'member_id': "99473492" })
    
    #mongo_groups = client.GetGroups({'lat': "51.326530", "lon" : "7.000413", "radius" : "100" })
    results = mongo_groups.results 
    
    print( "type: %s" % type( results ))
    print( "len: %d" % len( results ))
    
    MuGList = []
    
    pprint.pprint( results[0] )
    for k in results:
        MuGList.append( k )
            
    #print( "Total no of MUGs: %d" % len( MuGList) )
    d={}
    events = None 
    printed=False
    for k in MuGList :
        if k[ "organizer"]["name"] == "MongoDB" :
            #print( "Found: '%s'" % k['urlname'] )
            groups.insert_one( { "name" : k[ "urlname"], "info" : k } )
            events = client.GetEvents( { "group_id" : k[ 'id'] } ) #"member_id" : "99473492"})
            
            if events.__dict__[ 'meta']['count'] > 0 :
                #print( "Events for '%s'" % k[ 'urlname'])
                #pprint.pprint( events.__dict__ )
                
                if not printed:
                    members = client.GetMembers( { "group_id" : k['id']})
                    print( "Members of : '%s'" % k[ "urlname"]) 
                    d = members.__dict__  
                    printed = True
                    
    k = MuGList[ len( MuGList ) - 1 ]
    
    print( "Record================================>")
    print( "Record================================>")
    print( "Record================================>")
    print( "Record================================>")
    
    pprint.pprint( d.keys() )
    #print( "Events for : '%s'" % k[ "urlname"])
    #pprint.pprint( events.__dict__ )
#     for k,v in mongo_groups.__dict__.iteritems():
#         print( "k: %s" % k )
#         print( "v: %s" % v ) 




'''
Created on 12 Oct 2016

@author: jdrumgoole
'''

import pymongo
import bson
import pprint
from mugalyser.generator_utils import coroutine

class BatchWriter(object):
     
    def __init__(self, collection, transformFunc, newDocName, orderedWrites=False ):
         
        self._collection = collection
        self._orderedWrites = orderedWrites
        self._processFunc = transformFunc
        self._newDocName = newDocName
         


    '''
    Intialise coroutine automatically
    '''
    @coroutine 
    def bulkWrite(self, writeLimit=1000 ):
        bulker = None
        try : 
            if self._orderedWrites :
                bulker = self._collection.initialize_ordered_bulk_op()
            else:
                bulker = self._collection.initialize_unordered_bulk_op()
                
            bulkerCount = 0
            
            try :
                while True:
                    doc = (yield)
                    
                    #pprint.pprint( doc )) 
                    bulker.insert( self._processFunc(  self._newDocName, doc  ))
                    bulkerCount = bulkerCount + 1 
                    if ( bulkerCount == writeLimit ):
                        bulker.execute()
                        if self._orderedWrites :
                            bulker = self._collection.initialize_ordered_bulk_op()
                        else:
                            bulker = self._collection.initialize_unordered_bulk_op()
                             
                        bulkerCount = 0
            except GeneratorExit :
                if ( bulkerCount > 0 ) :
                    bulker.execute() 
            except bson.errors.InvalidDocument as e:
                print( "Invalid Document" )
                print( "bson.errors.InvalidDocument: %s" % e )

                raise
        except pymongo.errors.BulkWriteError as e :
            print( "Bulk write error : %s" % e.details )
            raise
        




'''
Created on 30 Sep 2016

@author: jdrumgoole
'''
from setuptools import setup
from mugalyser.version import __version__, __programName__
import os

pyfiles = [ f for f in os.listdir( "." ) if f.endswith( ".py" ) ]

    
setup(
    name = __programName__,
    version = __version__,
    
    author = "Joe Drumgoole",
    author_email = "joe@joedrumgoole.com",
    description = "MUGAlyser - a script to extract data from Meetup into a MongoDB database",
    long_description =
    '''
MUGLyser uses the meetup API to capture data into a MongoDB Database. The 
app/mugalyser_main.py program reads this data via the API. You will need a meetup API
key to read this data. If you have a Meetup pro account you can use the --pro option. 
You can analyse the data with apps/mug_analytics_main.py
''',

    license = "AGPL",
    keywords = "Meetup MUGS MongoDB API",
    url = "https://github.com/jdrumgoole/MUGAlyser",
    
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',


        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU Affero General Public License v3',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7' ],
   
    install_requires = [ "requests", "pymongo", "nose" ],
       
    packages = [ "mugalyser"],
    
    scripts  = [ "bin/mugalyser", "bin/meetupinfo", "bin/muginfo" ],

    test_suite='nose.collector',
    tests_require=['nose'],
)



'''
Created on 24 Mar 2017

@author: jdrumgoole
'''

from __future__ import print_function
import httplib2
import os

from apiclient import discovery
from apiclient.http import MediaFileUpload
from oauth2client import client
from oauth2client import tools
from oauth2client.file import Storage


# If modifying these scopes, delete your previously saved credentials
# at ~/.credentials/drive-python-quickstart.json
#SCOPES = 'https://www.googleapis.com/auth/drive.metadata.readonly'


homedir=os.getenv( "HOME")

CLIENT_SECRET_FILE = os.path.join(  homedir, 'pydrive_auth.json' )

class GDrive( object ):
    
    def __init__(self, application_name = "gdrive", client_secret_file=CLIENT_SECRET_FILE, credentials_dir=".credentials" ):
        self._client_secret_file = client_secret_file
        self._application_name = application_name
        self._scopes = 'https://www.googleapis.com/auth/drive.file'
        home_dir = os.path.expanduser('~')
        self._credential_dir = os.path.join(home_dir, credentials_dir )
        self._credentials_dir = credentials_dir
        if not os.path.exists( self._credential_dir):
            os.makedirs( self._credential_dir)
        self._credential_path = os.path.join( self._credential_dir,
                                              self._application_name + ".json" )
        
        self._secret_path = os.path.join( self._credential_dir, "pydrive_auth.json")
        
        self._credentials = None
        self._http = None
        self._service = None
        self.get_credentials()
        
    def get_credentials( self ):
        """Gets valid user credentials from storage.
    
        If nothing has been stored, or if the stored credentials are invalid,
        the OAuth2 flow is completed to obtain the new credentials.
    
        Returns:
            Credentials, the obtained credential.
        """
    
        store = Storage(self._credential_path)
        self._credentials = store.get()
        if not self._credentials or self._credentials.invalid:
            flow = client.flow_from_clientsecrets( self._secret_path, self._scopes )
            flow.user_agent = self._application_name
            self._credentials = tools.run_flow(flow, store )
            
        self._http = self._credentials.authorize(httplib2.Http())
        self._service = discovery.build('drive', 'v2', http=self._http)
        return self._credentials

    def upload_csvFile( self, folder_id, source_filename, target_filename=None ):
        return self.upload_file( folder_id, source_filename, 
                                 source_mimetype="text/csv", 
                                 target_mimetype="application/vnd.google-apps.spreadsheet", 
                                 target_filename=target_filename )
        
    def upload_file( self,  folder_id, source_filename, source_mimetype, target_mimetype, target_filename=None ):
        
        if target_filename is None :
            target_filename = os.path.basename( source_filename )
            target_filename = os.path.splitext( target_filename )[0]
            
        file_metadata = {
          'title' : target_filename,
          'mimeType' : target_mimetype,
          'parents': [{ 'id': folder_id }]
        }
        media = MediaFileUpload( source_filename,
                                 mimetype= source_mimetype,
                                 resumable=True)
        
        file_obj = self._service.files().insert(body=file_metadata,
                                                media_body=media,
                                                fields='id').execute()
    
        return ( target_filename, file_obj.get( "id"))
    
        #print( 'File ID: %s' % file_obj.get( "id"))

    def service(self):
        return self._service



MEETUP_API_KEY="AAAA"

import os

def get_meetup_key( default=MEETUP_API_KEY ):
    key = os.getenv( "MEETUP_API_KEY", default )
    if ( key == "AAAA" ) or ( len( key ) > 31 ) or len( key ) < 31 :
        raise ValueError( "Invalid API key for meetup : %s" % key )
    else:
        return key


'''
Created on 23 Nov 2016

@author: jdrumgoole
'''

from audit import Audit
from feedback import Feedback
import pprint

from enum import Enum

class Format( Enum ):
    oneline = 1
    summary = 2
    full = 3
    
def printCursor( c, filterField=None, filterList=None ):
    count = 0 
    for i in c :
        if filterField and filterList :
            if i[ filterField ]  in filterList:
                pprint.pprint( i )
        else:
            pprint.pprint( i )
        count = count + 1
    print( "Total records: %i" % count )
    
class MUGData( object ):
    
    def __init__( self, mdb, collection_name ):
        self._mdb = mdb
        self._audit = Audit( mdb )
        self._feedback = Feedback()
        self._collection = mdb.make_collection( collection_name )

    def collection(self):
        return self._collection
    
    @staticmethod
    def filter( cursor, selector, values ):
        for i in cursor:
            if i[ selector ] in values:
                yield i
        
    def find_one(self, query=None ):
        batch_query = { "batchID" : self._audit.getCurrentBatchID() }
        if query is not None:
            batch_query.update( query )
        
        #pprint.pprint( batch_query )
        return self._collection.find_one( batch_query )
        
    def find(self, q=None, *args, **kwargs ):
        
        query = { "batchID" : self._audit.getCurrentValidBatchID() } 
        if q :
            query.update( q )
            
        if args and kwargs :
            return self._collection.find( query, args, kwargs )
        elif args :
            return self._collection.find( query, args )
        elif kwargs:
            return self._collection.find( query, kwargs )
        else:
            return self._collection.find( query )

    
    def count(self, g ):
        count = 0
        for _ in g:
            count = count + 1
        return count
    
    def generator(self, cursor ):
        
        for i in cursor:
            yield i
            
    def summary(self, doc ):
        pass 
    
    def one_line(self, doc ):
        pass
    
    def full(self, doc ):
        pass
    
    def doc_print(self, doc, format_type = None  ):
        if format_type == "summary" :
            print( self.summary( doc ))
        elif format_type == "oneline" :
            print( self.one_line( doc ))
        else:
            pprint.pprint( doc )
            
    def count_print( self, iterator, format_type=None ):
        count = 0
        for i in iterator :
            count = count + 1
            self.doc_print( i, format_type  )
        print( "Total: %i" % count )


'''
Created on 11 Oct 2016

@author: jdrumgoole
'''

__version__       = "1.0.4alpha"
__programName__   = "MUGAlyser"
#
# Schema version 1.1 : Date mapped the new Group fields so they appear as date time.
#
__schemaVersion__ = "1.1"



'''
Created on 11 Oct 2016

@author: jdrumgoole
'''

from mugalyser.agg import Agg, Sorter
from mugalyser.feedback import Feedback
from mugalyser.mugdata import MUGData
import itertools
import pymongo

from mugalyser.audit import Audit
from utils.query import Query

class Members(MUGData):
    '''
    classdocs
    '''
    
    def __init__(self, mdb ):
        '''
        Constructor
        '''
        super( Members, self ).__init__( mdb, "members" ) 
        self._membersAgg = Agg( self._collection )
        self._membersAgg.addMatch({ "member.name": { "$exists" : 1 }})
        self._membersAgg.addProject( { "_id" : 0, "name" : "$member.name" })
        self._membersAgg.addGroup( { "_id" : "$name" , "occurences" : { "$sum" : 1 }})
        self._membersAgg.addSort( Sorter( occurences = pymongo.DESCENDING  )) # largest first
        self._memberCount = 0
        self._feedback = Feedback()
        self._audit = Audit( mdb )
        
    def get_group_members(self, url_name, q=None ):
        '''
        returns a MongoDB cursor.
        '''
        query = { "member.chapters" : { "$elemMatch" : { "urlname" : url_name }}}
        if q :
            query.update( q )

        return self.find( query )
        
    def get_many_group_members(self, groups, query=None):
        '''
        returns a generator
        '''
        
        return itertools.chain( *[ self.get_group_members( i, query ) for i in groups ] )

    def count_members(self, groups ):
        
        total = 0 
        for i in groups:
            count = self.get_group_members( i ).count()
            total = total + count
            
        return count
    
    def get_all_members(self, query=None ):
        '''
        Query meetup API for multiple groups.
        '''
        return self.find( query )
        
    def distinct_members(self ):
        return self._collection.distinct( "member.member_name")
    
    def get_by_name(self, name ):
        member = self.find_one( { "member.member_name" : name })
        
        if member is None:
            return None
        else:
            return member[ "member" ]
        
    def get_by_ID(self, member_id):
        val = self.find_one( { "member.member_id" : member_id })
        
        if val is None:
            return val
        else:
            return val[ "member" ]
        
    def get_by_join_date(self, start, end ):

        return self.find( { "member.join_time" : { "$gte" : start, "$lte" : end  }})
        
    def joined_by_year(self):
        
        agg_pipe = Agg( self._collection )    
        agg_pipe.addMatch( { "batchID" : self._audit.getCurrentBatchID() } )
        agg_pipe.addProject( { "_id" : 0, 
                               "member_id" : "$member.member_id",
                               "member_name" : "$member.member_name",
                               "year" : { "$year" : "$member.join_time" },
                              })
        agg_pipe.addGroup( { "_id" : "$year","total_registered" : { "$sum" : 1 }})
        
        return agg_pipe.aggregate()
        
        
    def get_members(self):
        return self._membersAgg.aggregate()
      
    def summary(self, doc ):
        return "name: %s, id: %s, country: %s" % ( doc[ "member" ][ "member_name"],
                                                    doc[ "member" ][ "member_id"],
                                                    doc[ "member" ][ "country" ] )
    
    def one_line(self, doc ):
        return "name : %s, id: %s" % ( doc[ "member"][ "member_name"],
                                       doc[ "member" ][ "member_id"] )

    
        
class Organizers( Members ):
    
    def get_organizers(self):
        return self.find( { "member.is_organizer" : True })
    
    def get_mugs(self, organizer_name  ):
        
        doc = self.find_one( { "member.is_organizer" : True, 
                               "member.member_name"  : organizer_name } )
        
        return doc["member"][ "chapters" ]
        
    def summary(self, doc ):
        m = doc[ "member"]
        
        header = " name: %s, id: %s, country: %s\n" % ( m[ "member_name"],
                                                        m[ "member_id"],
                                                        m[ "country" ] )
        
        for i in m[ "chapters" ]  :
            header = "%s \t%s\n" % ( header, i[ "urlname" ] ) 
            
        return header
            


'''
Created on 7 Oct 2016

@author: jdrumgoole
'''
from pprint import pprint

from mugalyser.mugdata import MUGData
import itertools

NORDICS_COUNTRIES = [ "Denmark", 
                      "Faroe Islands", 
                      "Finland", 
                      "Greenland", 
                      "Iceland", 
                      "Norway", 
                      "Sweden" ]
               
EU_COUNTRIES = [ "Austria", 
                 "Belgium", 
                 "Bulgaria", 
                 "Croatia", 
                 "Cyprus", 
                 "Czech Republic", 
                 "Denmark", 
                 "Estonia", 
                 "Finland", 
                 "France", 
                 "Germany", 
                 "Greece", 
                 "Hungary", 
                 "Ireland", 
                 "Italy", 
                 "Latvia", 
                 "Lithuania", 
                 "Luxembourg", 
                 "Malta", 
                 "Netherlands", 
                 "Poland", 
                 "Portugal", 
                 "Romania", 
                 "Slovakia", 
                 "Slovenia", 
                 "Spain", 
                 "Sweden", 
                 "United Kingdom" ]


class Groups(MUGData):
    '''
    classdocs
    '''

    def __init__(self, mdb ):
        
        super( Groups, self ).__init__( mdb, "groups")  

        
    def get_group(self, url_name ):
        return self.find_one( { "group.urlname": url_name })
    
    def get_all_groups(self, region=None):
        if region:
            if type( region ) is list:
                return self.find( { "group.country" : { "$in" : region }})
            else:
                raise ValueError( "region parameter is not a list (type = %s)" % type( region ))
        else:
            return self.find()
 
    def get_groups(self, group_names ):
        
        return itertools.chain( *[ self.get_group( i ) for i in group_names ] )
    
    def get_country_group_urlnames(self, country ):
        return self.get_region_group_urlnames( [ country ])
    
    def get_country(self, urlname ):
        
        group = self.find_one( { "group.urlname" : urlname } )
        return group[ "group"][ "country"]
    
    def get_region_group_urlnames(self, regions = None ):
        if regions:
            if type( regions ) is list:
                return [ x[ "group"]["urlname" ] for x in self.find( {  "group.country" : { "$in" : regions }}) ]
            else:
                raise ValueError( "regions parameter must be a list ( type=%s)" % type( regions ))  
        else:
            return [ x[ "group"]["urlname" ] for x in self.find() ]

            

    @staticmethod
    def summary( g ):
        return u"name: {0}\nmembers:{1}\ncountry: {2}\n".format( g[ "name" ], 
                                                                g[ "members" ], 
                                                                g[ "country" ])
    
    
    @staticmethod
    def printGroup( group, output="short" ):
        
        if output == "short" :
            print( group[ 'name' ] )
        elif output == "summary" :
            print( Groups.summary( group ))
        else:
            pprint( group )
        
    


'''
Created on 11 Oct 2016

@author: jdrumgoole
'''

from .audit import Audit

class UpcomingEventsx(object):
    '''
    classdocs
    '''
    
    def __init__(self, mdb ):
        '''
        Constructor
        '''
        self._mdb = mdb 
        self._events= mdb.upcomingEventsCollection()
        self._audit = Audit( mdb )
        
    def upcoming( self ):
        
        batchID = self._audit.getCurrentBatchID()
        #print( "Current Batch ID: %s" % lastBatchID )
        return self._events.find( { "batchID" : batchID})
            
    def summary(self, e ):
        
        return "name: {0}\ngroup: {1}\nrsvp:{2}\ntime: {3}\n".format(  e[ "name"], e[ "group"]["name"], e[ "yes_rsvp_count"], e[ "time" ])
    
        #return "name: '%s' group: '%s' rsvp: %s time: %s" % ( e[ "name"], e[ "group"]["name"], e[ "yes_rsvp_count"], e[ "time" ] )
    
    


'''
Created on 6 Sep 2016

@author: jdrumgoole
'''

import requests
import logging
import datetime
import time
import pprint

from copy import deepcopy

from mugalyser.apikey import get_meetup_key

from mugalyser.version import __programName__

def returnData( r ):
    if r.raise_for_status() is None:
        return ( r.headers, r.json())
        

class PaginatedRequest( object ):
    
    def __init__(self, items=500, logurl = False ):

        self._logurl = logurl
        self._items = 500
        
    def makeRequest(self, req, params ):
        logger = logging.getLogger( __programName__ )
    
        if self._logurl:
            print( "URL    : '%s'" % req )
            print( "params : '%s" % params )
            
        level = logger.getEffectiveLevel()
        
        logger.setLevel( logging.WARN ) # turn of info output for requests
        
        r = requests.get( req, params )
        logging.debug( "request URL   :'%s'", r.url )
        logging.debug( "request header: '%s'", r.headers )
        #print( "url: '%s'" % r.url )
        #print( "text: %s" % r.text )
        #print( "headers: %s" % r.headers )


        try:
            data = returnData( r )
            limit = int( data[0][ "X-RateLimit-Remaining"] )
            if  limit < 5 : #brute force, we can be more clever about this
                time.sleep( 1 )
                
            x={}
            
#             for k,v in data[1].items():
#     
#                 if type( v ) is unicode :
#                     x[ k ] = v
#                     print( "unicode")
#                 else:
#                     x[ k ] = str( v ).encode( 'utf8')
#                     print( "encoded")
#             #logging.debug( "return value: %s", data )
            return data
        
        except ValueError :
            print( "ValueError in makeRequests:")
            print( "request: '%s'" % r.url)
            print( "headers" )
            pprint.pprint( r.headers )
            print( "text" )
            pprint.pprint( r.text )
            raise
        
        except requests.HTTPError, e :
    
            logger.error( "HTTP Error  : %s:", e )
            raise
            
        finally:
            logger.setLevel( level )


            
    def getHref( self, s ):
        ( link, direction ) = s.split( ";", 2 )
        link = link[ 1:-1]
        ( _, direction ) = direction.split( "=", 2 )
        direction = direction[ 1:-1 ]
        return ( link, direction )
    
    def getNextPrev(self, header ):
        
        #headerDict = json.loads( header )
        link = header[ "Link" ]
        
        if "," in link : # has prev  and next fields
            ( nxt, prev ) = link.split( ",", 2 )
            ( nextLink, _ ) = self.getHref( nxt )
            ( prevLink, _ )  = self.getHref( prev )
        else:
            ( link, direction ) = self.getHref( link )
            #print( "direction: '%s'" % direction )
            if direction == "next" :
                nextLink = link
                prevLink = None
            else:
                prevLink = link
                nextLink = None
        
        return ( nextLink, prevLink )
    
    def paginatedRequest(self, req, params, reshaperFunc=None ):
        '''
        Takes a request and hands it off to the paginator API. It does this by initiating the request
        to get the first document back and then using it to look for headers.
        '''

        #print( "Intiate paginated request")    
        (header, body) = self.makeRequest( req, params )

        #print( "Paginator")
        #r = requests.get( self._api + url_name + "/events", params = params )
        #print( "request: '%s'" % r.url )
        #print( "header: '%s' )
        return self.paginator( header, body, params, func=reshaperFunc  )
    
    def paginator( self, headers, body, params, func=None, arg=None ):
        '''
        Meetup API returns results as pages. The old API embeds the 
        page data in a meta data object in the response object. The new API
        returns page data in the Header info. 
        
        Func is a function that takes a doc and returns a doc. Right now
        we use this to reshape geospatial coordinates into a format that MongoDB
        understands and to convert  meetup timestamps to datetime objects.
        
        paginator is a generator which yields results.
        
        '''
        
        #print( "paginatorEntry( %s )" % headers )
        if func is None:
            func = Reshaper.noop
            
        data = body
        #pprint.pprint( data )
        
        # old style format 
        if "meta" in body :
            for i in body[ "results"]:
                yield func( i )
        
            count = 0
            while ( data[ 'meta' ][ "next" ] != ""  ) :

                #print( "makeRequest (old): %i" % count )
                data = self.makeRequest( data['meta'][ 'next' ], params )[1]
                count = count + 1

            
                for i in data[ "results"]:
                    yield  func( i )
    
                    
        elif ( "Link" in headers ) : #new style pagination
            for i in data :
                yield func( i )
               
            count = 0 
            ( nxt, _) = self.getNextPrev(headers)
    
            count = 0
            while ( nxt is not None ) : # no next link in last page

                count = count + 1 
                #print( "make request (new): %i" % count )
                    
                ( headers, body ) = self.makeRequest( nxt, params )
                ( nxt, _ ) = self.getNextPrev(headers)
                for i in body :
                    yield  func( i )
    
        else: # new style but we have all the data
            for i in data:
                yield func( i )

def epochToDatetime( ts ):
    return datetime.datetime.fromtimestamp( ts /1000 )

class Reshaper( object ):
    
    def __init__(self ):
        self._reshape = {}
        
    def add( self, key, value ):
        self._reshape[ key] = value
        
    @staticmethod
    def noop( d, arg=None):
        return d
 
    def reshape( self, doc ) :
        newDoc = doc.copy()
        newDoc.update( self._reshape )
        return newDoc

    @staticmethod
    def reshapeGeospatial( doc ):
        doc[ "location" ] = { "type" : "Point", "coordinates": [ doc["lon"], doc["lat" ]] }
        del doc[ 'lat']
        del doc[ 'lon']
        return doc

    @staticmethod
    def reshapeMemberDoc( doc ):
        return Reshaper.reshapeTime( Reshaper.reshapeGeospatial(doc), [ "join_time", "last_access_time" ])

    @staticmethod
    def reshapeEventDoc( doc ):
        return Reshaper.reshapeTime( doc, [ "created", "updated", "time" ])
    
    @staticmethod
    def reshapeTime( doc, keys ):
        for i in keys:
            if i in doc :
                doc[ i ] =epochToDatetime( doc[ i ])
        
        return doc
    
    @staticmethod
    def reshapeGroupDoc( doc ):
        return Reshaper.reshapeTime( Reshaper.reshapeGeospatial(doc), 
                                     [ "created", "pro_join_date", "founded_date", "last_event" ])



    

        
    
def getHeaderLink( header ):
    #print( "getHeaderLink( %s)" % header )

    if "," in header:
        ( nxt, _ ) = header.split( ",", 2 )
    else:
        nxt = header
        
    ( link, relParam ) = nxt.split( ";", 2 )
    ( _, relVal ) = relParam.split( "=", 2 )
    link = link[1:-1] # strip angle brackets off link
    relVal = relVal[ 1:-1] # strip off quotes
    return ( link, relVal )


        
class MeetupAPI(object):
    '''
    classdocs
    '''
    @staticmethod
    def makeRequestURL( *args ):
        url = "https://api.meetup.com"
        
        for i in args:
            url = url + "/" + i
            
        return url
    
    def __init__(self, apikey = get_meetup_key(), items=500, logurl=False):
        '''
        Constructor
        '''
        
        self._api = "https://api.meetup.com/"
        self._params = {}
        self._params[ "key" ] = apikey
        self._items = str( items )
        self._requester = PaginatedRequest( self._items, logurl )
            
    def get_group(self, url_name ):
        
        params = deepcopy( self._params )
        
        return Reshaper.reshapeGroupDoc( self._requester.makeRequest( self._api + url_name, params = params )[1] ) 

    def get_past_events(self, url_name, items=20 ) :
        
        params = deepcopy( self._params )
        
        params[ "status" ]       = "past"
        params[ "page" ]         = self._items
        params[ "group_urlname"] = url_name
        
        return self._requester.paginatedRequest( self._api + "2/events", params, Reshaper.reshapeEventDoc )

    def get_all_attendees(self, groups=None, items=50 ):
        groupsIterator = None
        if groups :
            groupsIterator = groups
        else:
            groupsIterator = self.get_groups()
            
        for group in groupsIterator :
            return self.get_attendees(group, items)
        
    def get_attendees( self, url_name, items=50 ):
        
        for event in self.get_past_events( url_name ):
            #pprint( event )
            for attendee in self.get_event_attendees(event[ "id"], url_name, items ):
                yield ( attendee, event )
    
            
    def get_event_attendees(self, eventID, url_name, items=20 ):
        params = deepcopy( self._params )
        params[ "page" ] = self._items
        
        #https://api.meetup.com/DublinMUG/events/62760772/attendance?&sign=true&photo-host=public&page=20
        reqURL = self.makeRequestURL( url_name, "events", str( eventID ), "attendance")
        return self._requester.paginatedRequest( reqURL, params )
    
    def get_upcoming_events(self, url_name, items=20 ):
        
        params = deepcopy( self._params )
        params[ "status" ] = "upcoming"
        params[ "page"]    = self._items
        params[ "group_urlname"] = url_name
        
        return self._requester.paginatedRequest( self._api + "2/events", params, Reshaper.reshapeEventDoc)
    
    def get_member_by_id(self, member_id ):
        params = deepcopy( self._params )

        ( _, body ) = self._requester.makeRequest( self._api + "2/member/" + str( member_id ), params = params )
        
        return body
    
    def get_members(self, url_name, items=1000 ):
        
        params = deepcopy( self._params )
        params[ "group_urlname" ] = url_name
        params[ "page"]    = self._items

        return self._requester.paginatedRequest( self._api + "2/members", params )
    
    def get_groups(self, items=200 ):
        '''
        Get all groups associated with this API key.
        '''
        
        params = deepcopy( self._params )
        params[ "page" ]         = self._items
        logging.debug( "get_groups")
        return self._requester.paginatedRequest(self._api + "self/groups",  params )
    
    def get_pro_groups(self, items=20 ):
        '''
        Get all groups associated with this API key.
        '''
        
        params = deepcopy( self._params )
        params[ "page" ]         = self._items
        logging.debug( "get_pro_groups")
        
        return self._requester.paginatedRequest( self._api + "pro/MongoDB/groups", params, Reshaper.reshapeGroupDoc )
    
    def get_pro_members(self, items=200 ):
        params = deepcopy( self._params )
        params[ "page" ] = self._items
        logging.debug( "get_pro_members")
        
        return self._requester.paginatedRequest( self._api + "pro/MongoDB/members", params, Reshaper.reshapeMemberDoc)

    
    def get_pro_group_names( self ):
        for i in self.get_pro_groups() :
            yield i[ "urlname" ]
    


'''
Created on 22 Nov 2016

@author: jdrumgoole
'''
from pprint import pprint

def coroutine(func):
    def start( *args,**kwargs):
        cr = func(*args,**kwargs)
        cr.next()
        return cr

    return start

def printCount( iterator, printfunc=None ):
    count = 0
    for i in iterator :
        count = count + 1
        if printfunc is None:
            pprint( i )
        else:
            printfunc( i )
    print( "Total: %i" % count )